@@ -5,6 +5,8 @@ import inspect
 import json as json_module
 import logging
 import re
+from itertools import groupby
+
 import six
 
 from collections import namedtuple
@@ -26,7 +28,7 @@ except ImportError:
 try:
     from requests.packages.urllib3.connection import HTTPHeaderDict
 except ImportError:
-    from urllib3.connection import HTTPHeaderDict
+    from urllib3.response import HTTPHeaderDict
 
 if six.PY2:
     from urlparse import urlparse, parse_qsl, urlsplit, urlunsplit
@@ -53,6 +55,11 @@ except AttributeError:
     # Python 3.7
     Pattern = re.Pattern
 
+try:
+    from json.decoder import JSONDecodeError
+except ImportError:
+    JSONDecodeError = ValueError
+
 UNSET = object()
 
 Call = namedtuple("Call", ["request", "response"])
@@ -229,17 +236,45 @@ def _handle_body(body):
 _unspecified = object()
 
 
+def urlencoded_params_matcher(params):
+    def match(request_body):
+        return (
+            params is None
+            if request_body is None
+            else sorted(params.items()) == sorted(parse_qsl(request_body))
+        )
+
+    return match
+
+
+def json_params_matcher(params):
+    def match(request_body):
+        try:
+            if isinstance(request_body, bytes):
+                request_body = request_body.decode("utf-8")
+            return (
+                params is None
+                if request_body is None
+                else params == json_module.loads(request_body)
+            )
+        except JSONDecodeError:
+            return False
+
+    return match
+
+
 class BaseResponse(object):
     content_type = None
     headers = None
 
     stream = False
 
-    def __init__(self, method, url, match_querystring=_unspecified):
+    def __init__(self, method, url, match_querystring=_unspecified, match=[]):
         self.method = method
         # ensure the url has a default path set if the url is a string
         self.url = _ensure_url_default_path(url)
         self.match_querystring = self._should_match_querystring(match_querystring)
+        self.match = match
         self.call_count = 0
 
     def __eq__(self, other):
@@ -302,6 +337,13 @@ class BaseResponse(object):
         else:
             return False
 
+    def _body_matches(self, match, request_body):
+        for matcher in match:
+            if not matcher(request_body):
+                return False
+
+        return True
+
     def get_headers(self):
         headers = HTTPHeaderDict()  # Duplicate headers are legal
         if self.content_type is not None:
@@ -315,12 +357,15 @@ class BaseResponse(object):
 
     def matches(self, request):
         if request.method != self.method:
-            return False
+            return False, "Method does not match"
 
         if not self._url_matches(self.url, request.url, self.match_querystring):
-            return False
+            return False, "URL does not match"
 
-        return True
+        if not self._body_matches(self.match, request.body):
+            return False, "Parameters do not match"
+
+        return True, ""
 
 
 class Response(BaseResponse):
@@ -608,21 +653,33 @@ class RequestsMock(object):
     def _find_match(self, request):
         found = None
         found_match = None
+        match_failed_reasons = []
         for i, match in enumerate(self._matches):
-            if match.matches(request):
+            match_result, reason = match.matches(request)
+            if match_result:
                 if found is None:
                     found = i
                     found_match = match
                 else:
                     # Multiple matches found.  Remove & return the first match.
-                    return self._matches.pop(found)
+                    return self._matches.pop(found), match_failed_reasons
+            else:
+                match_failed_reasons.append(reason)
+        return found_match, match_failed_reasons
 
-        return found_match
+    def _parse_request_params(self, url):
+        params = {}
+        for key, val in groupby(parse_qsl(urlparse(url).query), lambda kv: kv[0]):
+            values = list(map(lambda x: x[1], val))
+            if len(values) == 1:
+                values = values[0]
+            params[key] = values
+        return params
 
     def _on_request(self, adapter, request, **kwargs):
-        match = self._find_match(request)
+        match, match_failed_reasons = self._find_match(request)
         resp_callback = self.response_callback
-        request.params = dict(parse_qsl(urlparse(request.path_url).query))
+        request.params = self._parse_request_params(request.path_url)
 
         if match is None:
             if any(
@@ -643,8 +700,10 @@ class RequestsMock(object):
                 "- %s %s\n\n"
                 "Available matches:\n" % (request.method, request.url)
             )
-            for m in self._matches:
-                error_msg += "- {} {}\n".format(m.method, m.url)
+            for i, m in enumerate(self._matches):
+                error_msg += "- {} {} {}\n".format(
+                    m.method, m.url, match_failed_reasons[i]
+                )
 
             response = ConnectionError(error_msg)
             response.request = request
